/*
 *   Copyright (c) International Business Machines  Corp., 2000
 *
 *   This program is free software;  you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY;  without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See
 *   the GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program;  if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *   MODULE_NAME:		fsckcbbl.c
 *
 *   COMPONENT_NAME: 	jfs utilities
 *
 *   FUNCTIONS:
 *              check_BdBlkLst_FillLevel
 *              ClrBdBlkLst_processing
 *              exec_clrbdblks
 *              examine_LVM_BadBlockLists
 *              get_LVM_BadBlockList_count
 *
 */

#define INCL_TYPES

/*
 * defines and includes common among the xfsck modules
 */
#include "xfsckint.h"

typedef ULONG LSNumber;
typedef LSNumber RelocationTable;   /* actually this is an array */

/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
 *
 * superblock buffer pointer
 *
 *      defined in xchkdsk.c
 */
extern struct superblock *sb_ptr;

/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
 *
 * fsck aggregate info structure pointer
 *
 *      defined in xchkdsk.c
 */
extern struct fsck_agg_record *agg_recptr;

/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
 *
 * For message processing
 *
 *      defined in xchkdsk.c
 */
extern char message_parm_0[];
extern char message_parm_1[];
extern char message_parm_2[];
extern char message_parm_3[];
extern char message_parm_4[];
extern char message_parm_5[];
extern char message_parm_6[];
extern char message_parm_7[];
extern char message_parm_8[];
extern char message_parm_9[];

extern char *msgprms[];
extern s16 msgprmidx[];

extern char *terse_msg_ptr;
extern char *verbose_msg_ptr;

extern char *MsgText[];

extern char *Vol_Label;

/* + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + + +
 *
 * Device information.
 *
 *      defined in xchkdsk.c
 */
extern HFILE  Dev_IOPort;
extern u32    Dev_blksize;


/* VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV
 *
 * The following are internal to this file
 *
 */

retcode_t exec_clrbdblks( void );

retcode_t examine_LVM_BadBlockLists( s32, s32 * );

retcode_t get_LVM_BadBlockList_count( s32 * );

/* VVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVVV */


/*****************************************************************************
 * NAME: check_BdBlkLst_FillLevel
 *
 * FUNCTION:	If any of the LVM Bad Block Tables composing the LVM Bad
 *		Block List for this partition is more than 50% full, issue a message.
 *
 * PARAMETERS:  none
 *
 * NOTES:	This routine is called only during autocheck processing.  fsck
 *		    cannot perform /B processing at that time since the file system
 *		    is not yet fully initialized.
 *
 * RETURNS:
 *      success: FSCK_OK
 *      failure: something else
 */
retcode_t check_BdBlkLst_FillLevel ( )
{
  retcode_t ccbblfl_rc = FSCK_OK;
  s32 intermed_rc = 0;
  s32 num_tables = 0;
  s32 highest_percent_full = 0;

  intermed_rc = get_LVM_BadBlockList_count( &num_tables );

  if ( intermed_rc == FSCK_OK ) {
    intermed_rc = examine_LVM_BadBlockLists( num_tables, &highest_percent_full );
    if ( intermed_rc == FSCK_OK ) {
      if ( highest_percent_full > 50 ) {
        fsck_send_msg( fsck_LVMFOUNDBDBLKS, 0, 0 );
      }
    }
  }

  return( ccbblfl_rc );
}  /* end of check_BdBlkLst_FillLevel() */


/*****************************************************************************
 * NAME: ClrBdBlkLst_processing
 *
 * FUNCTION:  Invoke the JFS processing to clear the LVM's bad block list.
 *
 * PARAMETERS:  none
 *
 * NOTES:	Starts a child process for the utility.
 *
 * RETURNS:
 *      success: FSCK_OK
 *      failure: something else
 */
retcode_t ClrBdBlkLst_processing ( )
{
  retcode_t kcbbl_rc = FSCK_OK;
  retcode_t intermed_rc = FSCK_OK;
  struct fsckcbbl_record  *cbblrecptr;
  int I_am_logredo = 0;

  cbblrecptr = &(agg_recptr->blkmp_ctlptr->cbblrec);
  if ( cbblrecptr == NULL ) {
    if ( agg_recptr->blkmp_ctlptr == NULL ) {
      intermed_rc = alloc_wrksp( sizeof(fsck_blk_map_hdr_t),
                                 dynstg_blkmap_hdr,
                                 I_am_logredo,
                                 (void **) &(agg_recptr->blkmp_ctlptr)
                               );
      if ( intermed_rc == FSCK_OK ) {
        strncpy( agg_recptr->blkmp_ctlptr->hdr.eyecatcher,
                 fbmh_eyecatcher_string,
                 strlen(fbmh_eyecatcher_string));  /* fill eyecatcher */
      }
    }
    intermed_rc = blkmap_get_ctl_page ( agg_recptr->blkmp_ctlptr );
    if ( intermed_rc == FSCK_OK ) {
      cbblrecptr = &(agg_recptr->blkmp_ctlptr->cbblrec);
    }
  }
  if ( cbblrecptr != NULL ) {
    memcpy( (void *) &(cbblrecptr->eyecatcher), (void *) "*unset**", 8 );
    intermed_rc = blkmap_put_ctl_page ( agg_recptr->blkmp_ctlptr );
  }

  fsck_send_msg( fsck_LVMFSNOWAVAIL, 0, 0 );
  fsck_send_msg( fsck_LVMTRNSBBLKSTOJFS, 0, 0 );

  /*
   * close the file system so clrbblks can get it
   */
  close_volume();

  kcbbl_rc = exec_clrbdblks( );

  /*
   * open the file system and get the clrbblks
   * communication area
   */
  open_volume( Vol_Label );
  intermed_rc = blkmap_get_ctl_page ( agg_recptr->blkmp_ctlptr );
  if ( intermed_rc == FSCK_OK ) {
    if ( memcmp((void *)&(cbblrecptr->eyecatcher),(void *)"*unset**", 8) != 0 ) {
      /*
       * The eyecatcher field was reset.  there is good reason to
       * believe that clrbblks processing did actually write to the
       * record.
       */
      sprintf( message_parm_0, "%d", cbblrecptr->cbbl_retcode );
      msgprms[0] = message_parm_0;
      msgprmidx[0] = 0;
      fsck_send_msg( fsck_CLRBBLKSRC, 0, 1 );

      sprintf( message_parm_0, "%d", cbblrecptr->LVM_lists );
      msgprms[0] = message_parm_0;
      msgprmidx[0] = 0;
      fsck_send_msg( fsck_CLRBBLVMNUMLISTS, 0, 1 );

      sprintf( message_parm_0, "%lld", (cbblrecptr->fs_last_metablk+1) );
      msgprms[0] = message_parm_0;
      msgprmidx[0] = 0;
      sprintf( message_parm_1, "%lld", (cbblrecptr->fs_first_wspblk-1) );
      msgprms[1] = message_parm_1;
      msgprmidx[1] = 0;
      fsck_send_msg( fsck_CLRBBRANGE, 0, 2 );

      sprintf( message_parm_0, "%d", cbblrecptr->reloc_extents );
      msgprms[0] = message_parm_0;
      msgprmidx[0] = 0;
      sprintf( message_parm_1, "%lld", cbblrecptr->reloc_blocks );
      msgprms[1] = message_parm_1;
      msgprmidx[1] = 0;
      fsck_send_msg( fsck_CLRBBACTIVITY, 0, 2 );

      sprintf( message_parm_0, "%d", cbblrecptr->total_bad_blocks );
      msgprms[0] = message_parm_0;
      msgprmidx[0] = 0;
      sprintf( message_parm_1, "%d", cbblrecptr->resolved_blocks );
      msgprms[1] = message_parm_1;
      msgprmidx[1] = 0;
      fsck_send_msg( fsck_CLRBBRESULTS, 0, 2 );
    }
  }

  return( kcbbl_rc );
}  /* end of ClrBdBlkLst_processing() */

